@@ -194,6 +194,13 @@ html_engine_save_output_string (HTMLEngineSaveState *state,
   return retval;
 }
 
+gboolean
+html_engine_save_output_buffer (HTMLEngineSaveState *state, const gchar *buffer, int len)
+{
+	if (len == -1)
+		len = strlen (buffer);
+	return state->receiver (state->engine, buffer, len, state->user_data);
+}
 
 
 static gchar *
@@ -214,6 +221,7 @@ get_body (HTMLEngine *e)
 	gchar *bg;
 	gchar *bg_image;
 	gchar *link;
+	gchar *margins, *lm, *rm, *tm, *bm;
 	gchar *url = NULL;
 
 	cset = e->settings->color_set;
@@ -226,8 +234,17 @@ get_body (HTMLEngine *e)
 		: g_strdup ("");
 	g_free (url);
 
-	body = g_strconcat ("<BODY", text, link, bg, bg_image, ">\n", NULL);
+	lm = e->leftBorder != LEFT_BORDER ? g_strdup_printf (" LEFTMARGIN=\"%d\"", e->leftBorder) : g_strdup ("");
+	rm = e->rightBorder != RIGHT_BORDER ? g_strdup_printf (" RIGHTMARGIN=\"%d\"", e->rightBorder) : g_strdup ("");
+	tm = e->topBorder != TOP_BORDER ? g_strdup_printf (" TOPMARGIN=\"%d\"", e->topBorder) : g_strdup ("");
+	bm = e->bottomBorder != BOTTOM_BORDER ? g_strdup_printf (" BOTTOMMARGIN=\"%d\"", e->bottomBorder) : g_strdup ("");
 
+	body = g_strconcat ("<BODY", text, link, bg, bg_image, lm, rm, tm, bm, ">\n", NULL);
+
+	g_free (lm);
+	g_free (rm);
+	g_free (tm);
+	g_free (bm);
 	g_free (text);
 	g_free (link);
 	g_free (bg);
@@ -313,6 +330,7 @@ html_engine_save (HTMLEngine *engine,
 	state.inline_frames = FALSE;
 	state.user_data = user_data;
 	state.last_level = 0;
+	state.nb_intervals = NULL;
 
 	if (! write_header (&state))
 		return FALSE;
@@ -348,6 +366,7 @@ html_engine_save_plain (HTMLEngine *engine,
 	state.inline_frames = FALSE;
 	state.user_data = user_data;
 	state.last_level = 0;
+	state.nb_intervals = NULL;
 
 	/* FIXME don't hardcode the length */
 	html_object_save_plain (engine->clue, &state, 72);
@@ -377,6 +396,11 @@ html_engine_save_buffer_free (HTMLEngineSaveState *state)
 	string = (GString *)state->user_data;
 
 	g_string_free (string, TRUE);
+
+	if (state->nb_intervals) {
+		g_slist_free (state->nb_intervals);
+		state->nb_intervals = NULL;
+	}
 	
 	g_free (state);
 }
@@ -392,6 +416,44 @@ html_engine_save_buffer_peek_text (HTMLEngineSaveState *state)
 	return string->str;
 }
 
+int
+html_engine_save_buffer_peek_text_len (HTMLEngineSaveState *state)
+{
+	GString *string;
+	
+	g_return_val_if_fail (state != NULL, 0);
+	string = (GString *)state->user_data;
+	
+	return string->len;
+}
+
+void
+html_engine_save_add_nb_interval (HTMLEngineSaveState *state, int start_offset, int end_offset)
+{
+	state->nb_intervals = g_slist_prepend (state->nb_intervals, GINT_TO_POINTER (end_offset));
+	state->nb_intervals = g_slist_prepend (state->nb_intervals, GINT_TO_POINTER (start_offset));
+}
+
+void
+html_engine_save_buffer_clear_line_breaks (HTMLEngineSaveState *state, PangoLogAttr *attrs)
+{
+	if (state->nb_intervals) {
+		GSList *cur;
+
+		for (cur = state->nb_intervals; cur; cur = cur->next->next) {
+			int i, start_offset, end_offset;
+			g_return_if_fail (cur->next);
+
+			start_offset = GPOINTER_TO_INT (cur->data);
+			end_offset = GPOINTER_TO_INT (cur->next->data);
+
+			for (i = start_offset; i < end_offset; i ++)
+				if (i > 0 && !attrs [i - 1].is_white)
+					attrs [i].is_line_break = 0;
+		}
+	}
+}
+
 HTMLEngineSaveState *
 html_engine_save_buffer_new (HTMLEngine *engine, gboolean inline_frames)
 {
@@ -405,6 +467,7 @@ html_engine_save_buffer_new (HTMLEngine *engine, gboolean inline_frames)
 		state->inline_frames = inline_frames;
 		state->user_data = (gpointer) g_string_new ("");
 		state->last_level = 0;
+		state->nb_intervals = NULL;
 	}
 
 	return state;
@@ -467,3 +530,21 @@ html_engine_save_get_paragraph_align (GtkHTMLParagraphAlignment align)
 
 	return NULL;
 }
+
+gint
+html_engine_save_string_append_nonbsp (GString *out, const guchar *s, guint length)
+{	
+	guint len = length;
+	
+	while (len--) {
+		if (IS_UTF8_NBSP (s)) {
+			g_string_append_c (out, ' ');
+			s += 2;
+			len--;
+		} else {
+			g_string_append_c (out, *s);
+			s++;
+		}
+	}
+	return length;
+}
