@@ -37,10 +37,6 @@
 #include "htmltokenizer.h"
 #include "htmlembedded.h"
 
-#define USE_SCROLLED_WINDOW 1
-#ifndef USE_SCROLLED_WINDOW
-#include <gal/widgets/e-scroll-frame.h>
-#endif
 
 HTMLIFrameClass html_iframe_class;
 static HTMLEmbeddedClass *parent_class = NULL;
@@ -61,7 +57,8 @@ iframe_url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpo
 	HTMLIFrame *iframe = HTML_IFRAME (data);
 	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(iframe)->parent);
 
-	g_signal_emit_by_name (parent->engine, "url_requested", url, handle);
+	if (!html->engine->stopped)
+		g_signal_emit_by_name (parent->engine, "url_requested", url, handle);
 }
 
 static void
@@ -145,7 +142,7 @@ set_max_width (HTMLObject *o, HTMLPainter *painter, gint max_width)
 	HTMLEngine *e = GTK_HTML (HTML_IFRAME (o)->html)->engine;
 
 	o->max_width = max_width;
-	html_object_set_max_width (e->clue, e->painter, max_width - e->leftBorder - e->rightBorder);
+	html_object_set_max_width (e->clue, e->painter, max_width - (html_engine_get_left_border (e) + html_engine_get_right_border (e)));
 }
 
 static void
@@ -158,15 +155,6 @@ reset (HTMLObject *o)
 	html_object_reset (GTK_HTML (iframe->html)->engine->clue);
 }
 
-static void
-draw_background (HTMLObject *self,
-		 HTMLPainter *p,
-		 gint x, gint y,
-		 gint width, gint height,
-		 gint tx, gint ty)
-{
-}
-
 /* FIXME rodo - draw + set_painter is not much clean now, needs refactoring */
 
 static void
@@ -188,9 +176,9 @@ draw (HTMLObject *o,
 
 		html_object_draw (e->clue, e->painter,
 				  x, y,
-				  width - pixel_size * (e->leftBorder + e->rightBorder),
-				  height - pixel_size * (e->topBorder + e->bottomBorder),
-				  tx + pixel_size * e->leftBorder, ty + pixel_size * e->topBorder);
+				  width - pixel_size * (html_engine_get_left_border (e) + html_engine_get_right_border (e)),
+				  height - pixel_size * (html_engine_get_top_border (e) + html_engine_get_bottom_border (e)),
+				  tx + pixel_size * html_engine_get_left_border (e), ty + pixel_size * html_engine_get_top_border (e));
 	} else
 		(*HTML_OBJECT_CLASS (parent_class)->draw) (o, p, x, y, width, height, tx, ty);
 }
@@ -223,19 +211,19 @@ forall (HTMLObject *self,
 }
 
 static gint
-check_page_split (HTMLObject *self, gint y)
+check_page_split (HTMLObject *self, HTMLPainter *p, gint y)
 {
-	gint y1, y2;
 	HTMLEngine *e = GTK_HTML (HTML_IFRAME (self)->html)->engine;
+	int y1, y2, pixel_size = html_painter_get_pixel_size (p);
 
-	y1 = self->y - self->ascent + e->topBorder;
-	y2 = self->y + self->descent + e->topBorder;
+	y1 = self->y - self->ascent + pixel_size*html_engine_get_top_border (e);
+	y2 = self->y + self->descent + pixel_size*html_engine_get_bottom_border (e);
 	
 	if (y1 > y) 
 		return 0;
 
 	if (y >= y1 && y < y2)
-		return html_object_check_page_split (e->clue, y - y1) + y1;
+		return html_object_check_page_split (e->clue, p, y - y1) + y1;
 	
 	return y;
 }
@@ -310,17 +298,12 @@ html_iframe_set_margin_height (HTMLIFrame *iframe, gint margin_height)
 void
 html_iframe_set_scrolling (HTMLIFrame *iframe, GtkPolicyType scroll)
 {
-#if USE_SCROLLED_WINDOW
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (iframe->scroll),
 					scroll, scroll);
-#else
-	e_scroll_frame_set_policy (E_SCROLL_FRAME (iframe->scroll),
-				   scroll, scroll);
-#endif					
 }
 
 static gboolean
-calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
+html_iframe_real_calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 {
 	HTMLIFrame *iframe;
 	HTMLEngine *e;
@@ -349,6 +332,8 @@ calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 		
 		html_iframe_set_scrolling (iframe, GTK_POLICY_NEVER);
 
+		e->width = width;
+		e->height = height;
 		o->width = width;
 		o->ascent = height;
 		o->descent = 0;
@@ -482,8 +467,11 @@ save (HTMLObject *s,
 	if (state->inline_frames && e->clue) {
 		buffer = html_engine_save_buffer_new (e, state->inline_frames);
 		html_object_save (e->clue, buffer);
+
 		if (state->error || 
-		    !html_engine_save_output_string (state, html_engine_save_buffer_peek_text (buffer))) {
+		    !html_engine_save_output_buffer (state,
+						     html_engine_save_buffer_peek_text (buffer),
+						     html_engine_save_buffer_peek_text_len (buffer))) {
 			html_engine_save_buffer_free (buffer);
 			return FALSE;
 		}
@@ -534,7 +522,9 @@ save_plain (HTMLObject *s,
 		buffer = html_engine_save_buffer_new (e, state->inline_frames);
 		html_object_save_plain (e->clue, buffer, requested_width);
 		if (state->error || 
-		    !html_engine_save_output_string (state, html_engine_save_buffer_peek_text (buffer))) {
+		    !html_engine_save_output_buffer (state,
+						     html_engine_save_buffer_peek_text (buffer),
+						     html_engine_save_buffer_peek_text_len (buffer))) {
 			html_engine_save_buffer_free (buffer);
 			return FALSE;
 		}
@@ -575,7 +565,6 @@ html_iframe_init (HTMLIFrame *iframe,
 	GtkWidget *new_widget;
 	GtkHTML   *new_html;
 	GtkHTML   *parent_html;
-	GtkHTMLStream *handle;
 	GtkWidget *scrolled_window;
 	gint depth;
 
@@ -585,14 +574,9 @@ html_iframe_init (HTMLIFrame *iframe,
 	html_embedded_init (em, HTML_EMBEDDED_CLASS (klass),
 			    parent, NULL, NULL);
 	
-#if USE_SCROLLED_WINDOW
 	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-#else
-	scrolled_window = e_scroll_frame_new (NULL, NULL);
-	e_scroll_frame_set_shadow_type (E_SCROLL_FRAME (scrolled_window), 
-					border ? GTK_SHADOW_IN : GTK_SHADOW_NONE);
-
-#endif
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
+					     border ? GTK_SHADOW_IN : GTK_SHADOW_NONE);
 	/*
 	 * FIXME
 	 * are we missing:
@@ -604,6 +588,7 @@ html_iframe_init (HTMLIFrame *iframe,
 
 	new_widget = gtk_html_new ();
 	new_html = GTK_HTML (new_widget);
+	new_html->engine->cursor_hide_count = 0;
 
 	new_tokenizer = html_tokenizer_clone (parent_html->engine->ht);
 
@@ -624,12 +609,21 @@ html_iframe_init (HTMLIFrame *iframe,
 	iframe->gdk_painter = NULL;
 	iframe->frameborder = border;
 	gtk_html_set_base (new_html, src);
+	g_signal_connect (new_html, "url_requested", G_CALLBACK (iframe_url_requested), iframe);
 
-	handle = gtk_html_begin (new_html);
+	if (depth < 10) {
+		if (parent_html->engine->stopped) {
+			gtk_html_stop (new_html);
+			gtk_html_load_empty (new_html);
+		} else {
+			GtkHTMLStream *handle;
 
+			handle = gtk_html_begin (new_html);
+			g_signal_emit_by_name (parent_html->engine, "url_requested", src, handle);
+		}
+	}
 	new_html->engine->clue->parent = HTML_OBJECT (iframe);
 	
-	g_signal_connect (new_html, "url_requested", G_CALLBACK (iframe_url_requested), iframe);
 #if 0
 	/* NOTE: because of peculiarities of the frame/gtkhtml relationship
 	 * on_url and link_clicked are emitted from the toplevel widget not
@@ -650,8 +644,6 @@ html_iframe_init (HTMLIFrame *iframe,
 	  gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
 	  GTK_SIGNAL_FUNC (iframe_button_press_event), iframe);
 	*/
-	if (depth < 10)
-		g_signal_emit_by_name (parent_html->engine, "url_requested", src, handle);
 	
 	gtk_widget_set_size_request (scrolled_window, width, height);
 
@@ -704,7 +696,7 @@ html_iframe_class_init (HTMLIFrameClass *klass,
 	object_class->destroy                 = destroy;
 	object_class->save                    = save;
 	object_class->save_plain              = save_plain;
-	object_class->calc_size               = calc_size;
+	object_class->calc_size               = html_iframe_real_calc_size;
 	object_class->calc_min_width          = calc_min_width;
 	object_class->set_painter             = set_painter;
 	object_class->reset                   = reset;
@@ -720,7 +712,6 @@ html_iframe_class_init (HTMLIFrameClass *klass,
 	object_class->get_engine              = get_engine;
 	object_class->check_point             = check_point;
 	object_class->is_container            = is_container;
-	object_class->draw_background         = draw_background;
 	object_class->append_selection_string = append_selection_string;
 	
 	embedded_class->reparent = reparent;
