@@ -49,6 +49,7 @@
 #include "htmlprinter.h"
 #include "htmlgdkpainter.h"
 #include "htmlplainpainter.h"
+#include "htmlsettings.h"
 
 /* HTMLImageFactory stuff.  */
 
@@ -350,18 +351,27 @@ static void
 draw_plain (HTMLObject *o, HTMLPainter *p, gint x, gint y, gint width, gint height, gint tx, gint ty)
 {
 	HTMLImage *img = HTML_IMAGE (o);
+	HTMLEngine *e;
+
+	if (p->widget && GTK_IS_HTML (p->widget))
+		e = GTK_HTML (p->widget)->engine;
+	else
+		return;
 
 	if (img->alt && *img->alt) {
 
 		/* FIXME: cache items and glyphs? */
 		if (o->selected) {
 			html_painter_set_pen (p, &html_colorset_get_color_allocated
-					      (p, p->focus ? HTMLHighlightColor : HTMLHighlightNFColor)->color);
+					      (e->settings->color_set, p,
+					       p->focus ? HTMLHighlightColor : HTMLHighlightNFColor)->color);
 			html_painter_fill_rect (p, o->x + tx, o->y + ty - o->ascent, o->width, o->ascent + o->descent);
 			html_painter_set_pen (p, &html_colorset_get_color_allocated
-					      (p, p->focus ? HTMLHighlightTextColor : HTMLHighlightTextNFColor)->color);
+					      (e->settings->color_set, p,
+					       p->focus ? HTMLHighlightTextColor : HTMLHighlightTextNFColor)->color);
 		} else { 
-			html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLTextColor)->color);
+			html_painter_set_pen (p, &html_colorset_get_color_allocated (e->settings->color_set, p,
+										     HTMLTextColor)->color);
 		}
 		html_painter_draw_text (p, o->x + tx, o->y + ty, img->alt, g_utf8_strlen (img->alt, -1), NULL, NULL, NULL, 0, 0);
 	}
@@ -373,6 +383,12 @@ draw_focus  (HTMLPainter *painter, GdkRectangle *box)
 	HTMLGdkPainter *p;
 	GdkGCValues values;
 	gchar dash [2];
+	HTMLEngine *e;
+
+	if (painter->widget && GTK_IS_HTML (painter->widget))
+		e = GTK_HTML (painter->widget)->engine;
+	else
+		return;
 
 	if (HTML_IS_PRINTER (painter))
 		return;
@@ -380,7 +396,8 @@ draw_focus  (HTMLPainter *painter, GdkRectangle *box)
 	p = HTML_GDK_PAINTER (painter);
 	/* printf ("draw_image_focus\n"); */
 
-	gdk_gc_set_foreground (p->gc, &html_colorset_get_color_allocated (painter, HTMLTextColor)->color);
+	gdk_gc_set_foreground (p->gc, &html_colorset_get_color_allocated (e->settings->color_set,
+									  painter, HTMLTextColor)->color);
 	gdk_gc_get_values (p->gc, &values);
 
 	dash [0] = 1;
@@ -406,6 +423,12 @@ draw (HTMLObject *o,
 	GdkColor *highlight_color;
 	guint pixel_size;
 	GdkRectangle paint;
+	HTMLEngine *e;
+
+	if (painter->widget && GTK_IS_HTML (painter->widget))
+		e = GTK_HTML (painter->widget)->engine;
+	else
+		return;
 
 	/* printf ("Image::draw\n"); */
 
@@ -436,7 +459,8 @@ draw (HTMLObject *o,
 
 	if (o->selected) {
 		highlight_color = &html_colorset_get_color_allocated
-			(painter, painter->focus ? HTMLHighlightColor : HTMLHighlightNFColor)->color;
+			(e->settings->color_set, painter,
+			 painter->focus ? HTMLHighlightColor : HTMLHighlightNFColor)->color;
 	} else
 		highlight_color = NULL;
 
@@ -461,7 +485,7 @@ draw (HTMLObject *o,
 						o->ascent + o->descent - 2 * vspace);
 		}
 		html_painter_draw_panel (painter,
-					 &((html_colorset_get_color (painter->color_set, HTMLBgColor))->color),
+					 &((html_colorset_get_color (e->settings->color_set, HTMLBgColor))->color),
 					 o->x + tx + hspace,
 					 o->y + ty - o->ascent + vspace,
 					 o->width - 2 * hspace,
@@ -507,7 +531,7 @@ draw (HTMLObject *o,
 		}
 		
 		html_painter_draw_panel (painter,
-					 &((html_colorset_get_color (painter->color_set, HTMLBgColor))->color),
+					 &((html_colorset_get_color (e->settings->color_set, HTMLBgColor))->color),
 					 base_x - image->border * pixel_size,
 					 base_y - image->border * pixel_size,
 					 scale_width + (2 * image->border) * pixel_size,
@@ -593,7 +617,7 @@ save (HTMLObject *self,
 		return FALSE;	
 
 	if (image->percent_width) {
-		if (!html_engine_save_output_string (state, " WIDTH=\"%d\%\"", image->specified_width))
+		if (!html_engine_save_output_string (state, " WIDTH=\"%d%%\"", image->specified_width))
 			return FALSE;
 	} else if (image->specified_width > 0) {
 		if (!html_engine_save_output_string (state, " WIDTH=\"%d\"", image->specified_width))
@@ -601,7 +625,7 @@ save (HTMLObject *self,
 	}
 
 	if (image->percent_height) {
-		if (!html_engine_save_output_string (state, " HEIGHT=\"%d\%\"", image->specified_height))
+		if (!html_engine_save_output_string (state, " HEIGHT=\"%d%%\"", image->specified_height))
 			return FALSE;
 	} else if (image->specified_height > 0) {
 		if (!html_engine_save_output_string (state, " HEIGHT=\"%d\"", image->specified_height))
@@ -1104,7 +1128,7 @@ html_image_factory_end_pixbuf (GtkHTMLStream *stream,
 
 	update_or_redraw (ip);
 	if (ip->factory->engine->opened_streams && ip->factory->engine->block_images)
-		ip->factory->engine->opened_streams --;
+		html_engine_opened_streams_decrement (ip->factory->engine);
 	/* printf ("IMAGE(%p) opened streams: %d\n", ip->factory->engine, ip->factory->engine->opened_streams); */
 	if (ip->factory->engine->opened_streams == 0 && ip->factory->engine->block && ip->factory->engine->block_images)
 		html_engine_schedule_update (ip->factory->engine);
@@ -1153,7 +1177,9 @@ html_image_pointer_update (HTMLImagePointer *ip)
 	for (cur = ip->interests; cur; cur = cur->next) {
 		HTMLImage           *image = cur->data;
 		
-		if (image && image->animation_active) {
+		if (image && image->animation_active && html_object_is_parent (engine->clue, HTML_OBJECT (image))) {
+			DA (printf ("queue draw (%p)\n", image);)
+
 			image->animation_active = FALSE;
 			html_engine_queue_draw (engine, HTML_OBJECT (image));
 		}
@@ -1383,7 +1409,7 @@ html_image_pointer_load (HTMLImagePointer *ip)
 	html_image_pointer_ref (ip);
 
 	if (ip->factory->engine->block_images)
-		ip->factory->engine->opened_streams ++;
+		html_engine_opened_streams_increment (ip->factory->engine);
 	return gtk_html_stream_new (GTK_HTML (ip->factory->engine->widget),
 				    html_image_factory_types,
 				    html_image_factory_write_pixbuf,
@@ -1565,14 +1591,16 @@ html_image_factory_deactivate_animations (HTMLImageFactory *factory)
 static void
 ref_image_ptr (gpointer key, gpointer val, gpointer data)
 {
-	html_image_pointer_ref (HTML_IMAGE_POINTER (val));
+	if (HTML_IMAGE_POINTER (val)->animation)
+		html_image_pointer_ref (HTML_IMAGE_POINTER (val));
 	/* printf ("ref(%p) %s --> %d\n", val, HTML_IMAGE_POINTER (val)->url, HTML_IMAGE_POINTER (val)->refcount); */
 }
 
 static void
 unref_image_ptr (gpointer key, gpointer val, gpointer data)
 {
-	html_image_pointer_unref (HTML_IMAGE_POINTER (val));
+	if (HTML_IMAGE_POINTER (val)->animation)
+		html_image_pointer_unref (HTML_IMAGE_POINTER (val));
 }
 
 void
