@@ -610,6 +610,8 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, HT
 	double width, asc, dsc;
 	gboolean temp_pi = FALSE;
 
+	width = 0.0;
+
 	printer = HTML_PRINTER (painter);
 	g_return_val_if_fail (printer->context != NULL, 0);
 
@@ -633,16 +635,14 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, HT
 		const gchar *c_text = text;
 		PangoAttrIterator *iter = NULL;
 
-		if (attrs)
-			iter = pango_attr_list_get_iterator (attrs);
-
 		html_printer_coordinates_to_gnome_print (printer, x, y, &print_x, &print_y);
 		c_text = text;
 		for (gl = glyphs; gl && char_offset < len; gl = gl->next) {
 			GnomeFont *c_font;
 			GdkGC *gc, *bg_gc;
 			gboolean underline, strikethrough, bgcolor;
-			gint cw = 0, c_bytes, begin, end;
+			gint c_bytes, begin, end;
+			double cw = 0.0;
 
 			str = (PangoGlyphString *) gl->data;
 			gl = gl->next;
@@ -653,25 +653,23 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, HT
 			c_bytes = g_utf8_offset_to_pointer (c_text, str->num_glyphs) - c_text;
 			gnome_print_gsave (printer->context);
 			bgcolor = underline = strikethrough = FALSE;
-			process_attrs (printer, pi->entries [ii].item->analysis.extra_attrs, &style, &bgcolor, &underline, &strikethrough, TRUE);
+
+			if (attrs)
+				iter = pango_attr_list_get_iterator (attrs);
 
 			if (iter) {
-				pango_attr_iterator_range (iter, &begin, &end);
-				while (begin < c_text - text) {
-					if (!pango_attr_iterator_next (iter)) {
-						pango_attr_iterator_destroy (iter);
-						iter = NULL;
-						break;
-					}
+				do {
 					pango_attr_iterator_range (iter, &begin, &end);
-				}
-				if (iter && begin < (c_text - text) + c_bytes) {
-					GSList *attr_list;
-					attr_list = pango_attr_iterator_get_attrs (iter);
-					process_attrs (printer, attr_list, &style, &bgcolor, &underline, &strikethrough, TRUE);
-					html_text_free_attrs (attr_list);
-				}
+					if (iter && MAX (begin, c_text - text) < MIN (end, (c_text - text) + c_bytes)) {
+						GSList *attr_list;
+						attr_list = pango_attr_iterator_get_attrs (iter);
+						process_attrs (printer, attr_list, &style, &bgcolor, &underline, &strikethrough, TRUE);
+						html_text_free_attrs (attr_list);
+					}
+				} while (pango_attr_iterator_next (iter));
+				pango_attr_iterator_destroy (iter);
 			}
+			process_attrs (printer, pi->entries [ii].item->analysis.extra_attrs, &style, &bgcolor, &underline, &strikethrough, TRUE);
 			if (style != GTK_HTML_FONT_STYLE_DEFAULT)
 				c_font = html_painter_get_font (HTML_PAINTER (printer), HTML_PAINTER (printer)->font_face, style);
 
@@ -722,8 +720,6 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, HT
 			c_text += c_bytes;
 			char_offset += str->num_glyphs;
 		}
-		if (iter)
-			pango_attr_iterator_destroy (iter);
 	}
 
 	if (temp_pi) {
@@ -778,6 +774,7 @@ text_size (HTMLPainter *painter, const gchar *text, guint blen, HTMLTextPangoInf
 {
 	HTMLPrinter *printer;
 	gboolean temp_pi = FALSE;
+	double real_width = 0.0;
 
 	printer = HTML_PRINTER (painter);
 	g_return_if_fail (printer->context != NULL);
@@ -787,8 +784,6 @@ text_size (HTMLPainter *painter, const gchar *text, guint blen, HTMLTextPangoInf
 		*asc = SCALE_GNOME_PRINT_TO_ENGINE (gnome_font_get_ascender (font));
 	if (dsc)
 		*dsc = SCALE_GNOME_PRINT_TO_ENGINE (-gnome_font_get_descender (font));
-	if (width)
-		*width = 0;
 
 	if (!pi) {
 		pi = html_painter_text_itemize_and_prepare_glyphs (painter, html_painter_get_font (painter, painter->font_face, painter->font_style),
@@ -805,9 +800,6 @@ text_size (HTMLPainter *painter, const gchar *text, guint blen, HTMLTextPangoInf
 		PangoGlyphString *str;
 		PangoAttrIterator *iter = NULL;
 
-		if (attrs)
-			iter = pango_attr_list_get_iterator (attrs);
-
 		c_text = text;
 		for (gl = glyphs; gl && char_offset < blen; gl = gl->next) {
 			GnomeFont *c_font;
@@ -819,25 +811,24 @@ text_size (HTMLPainter *painter, const gchar *text, guint blen, HTMLTextPangoInf
 			style = GTK_HTML_FONT_STYLE_DEFAULT;
 			c_font = font;
 			c_bytes = g_utf8_offset_to_pointer (c_text, str->num_glyphs) - c_text;
-			process_attrs (printer, pi->entries [ii].item->analysis.extra_attrs, &style, NULL, NULL, NULL, FALSE);
+
+			if (attrs)
+				iter = pango_attr_list_get_iterator (attrs);
 
 			if (iter) {
-				pango_attr_iterator_range (iter, &begin, &end);
-				while (begin < c_text - text) {
-					if (!pango_attr_iterator_next (iter)) {
-						pango_attr_iterator_destroy (iter);
-						iter = NULL;
-						break;
-					}
+				do {
 					pango_attr_iterator_range (iter, &begin, &end);
-				}
-				if (iter && begin < (c_text - text) + c_bytes) {
-					GSList *attr_list;
-					attr_list = pango_attr_iterator_get_attrs (iter);
-					process_attrs (printer, attr_list, &style, NULL, NULL, NULL, FALSE);
-					html_text_free_attrs (attr_list);
-				}
+					if (iter && MAX (begin, c_text - text) < MIN (end, (c_text - text) + c_bytes)) {
+						GSList *attr_list;
+						attr_list = pango_attr_iterator_get_attrs (iter);
+						process_attrs (printer, attr_list, &style, NULL, NULL, NULL, FALSE);
+						html_text_free_attrs (attr_list);
+					}
+				} while (pango_attr_iterator_next (iter));
+				pango_attr_iterator_destroy (iter);
 			}
+
+			process_attrs (printer, pi->entries [ii].item->analysis.extra_attrs, &style, NULL, NULL, NULL, FALSE);
 			if (style != GTK_HTML_FONT_STYLE_DEFAULT)
 				c_font = html_painter_get_font (HTML_PAINTER (printer), HTML_PAINTER (printer)->font_face, style);
 
@@ -847,12 +838,10 @@ text_size (HTMLPainter *painter, const gchar *text, guint blen, HTMLTextPangoInf
 				*dsc = MAX (*dsc, SCALE_GNOME_PRINT_TO_ENGINE (-gnome_font_get_descender (c_font)));
 
 			if (width)
-				*width += gnome_font_get_width_utf8_sized (c_font, c_text, c_bytes);
+				real_width += gnome_font_get_width_utf8_sized (c_font, c_text, c_bytes);
 			c_text += c_bytes;
 			char_offset += str->num_glyphs;
 		}
-		if (iter)
-			pango_attr_iterator_destroy (iter);
 	}
 
 	if (temp_pi) {
@@ -862,7 +851,8 @@ text_size (HTMLPainter *painter, const gchar *text, guint blen, HTMLTextPangoInf
 			html_text_pango_info_destroy (pi);
 	}
 
-	*width = SCALE_GNOME_PRINT_TO_ENGINE (*width);
+	if (width)
+		*width = SCALE_GNOME_PRINT_TO_ENGINE (real_width);
 }
 
 static void
@@ -910,7 +900,7 @@ alloc_font (HTMLPainter *painter, gchar *face, gdouble size, gboolean points, Gt
 	gboolean italic;
 
 	weight = (style & GTK_HTML_FONT_STYLE_BOLD) ? GNOME_FONT_BOLD : GNOME_FONT_BOOK;
-	italic = (style & GTK_HTML_FONT_STYLE_ITALIC);
+	italic = (style & GTK_HTML_FONT_STYLE_ITALIC) ? TRUE : FALSE;
 
 	font = gnome_font_find_closest_from_weight_slant (face ? face : (style & GTK_HTML_FONT_STYLE_FIXED ? "Monospace" : "Sans"),
 							  weight, italic, get_font_size (printer, points, size));
