@@ -1005,7 +1005,9 @@ calc_row_heights (HTMLTable *table,
 					ROW_HEIGHT (table, rl) = height;
 			}
 		}
+		/* printf ("height %d: %d\n", r, ROW_HEIGHT (table, r)); */
 	}
+	/* printf ("height %d: %d\n", r, ROW_HEIGHT (table, r)); */
 }
 
 static void
@@ -1037,6 +1039,7 @@ html_table_set_cells_position (HTMLTable *table, HTMLPainter *painter)
 				HTML_OBJECT (cell)->x = COLUMN_OPT (table, c) + pixel_size * border_extra;
 				HTML_OBJECT (cell)->y = ROW_HEIGHT (table, rl) + pixel_size * (- table->spacing)
 					- HTML_OBJECT (cell)->descent;
+				/* printf ("y: %d\n", HTML_OBJECT (cell)->y); */
 				html_object_set_max_height (HTML_OBJECT (cell), painter,
 							    ROW_HEIGHT (table, rl) - ROW_HEIGHT (table, cell->row)
 							    - pixel_size * (table->spacing + border_extra));
@@ -1295,7 +1298,8 @@ calc_min_width (HTMLObject *o,
 {
 	HTMLTable *table = HTML_TABLE (o);
 
-	calc_column_width_template (table, painter, table->columnMin, html_object_calc_min_width, table->columnMin);
+	calc_column_width_template (table, painter, table->columnPref, html_object_calc_preferred_width, table->columnPref);
+	calc_column_width_template (table, painter, table->columnMin, html_object_calc_min_width, table->columnPref);
 
 	return o->flags & HTML_OBJECT_FLAG_FIXEDWIDTH
 		? MAX (html_painter_get_pixel_size (painter) * table->specified_width,
@@ -1308,15 +1312,17 @@ calc_preferred_width (HTMLObject *o,
 		      HTMLPainter *painter)
 {
 	HTMLTable *table = HTML_TABLE (o);
+	int min_width;
+
+	/* note that calculating min width prepares columnPref for us */
+	min_width = html_object_calc_min_width (o, painter);
 
-	calc_column_width_template (table, painter, table->columnPref,
-				    html_object_calc_preferred_width, table->columnPref);
 	calc_column_width_template (table, painter, table->columnFixed,
 				    (gint (*)(HTMLObject *, HTMLPainter *)) html_table_cell_get_fixed_width,
 				    table->columnPref);
 
 	return o->flags & HTML_OBJECT_FLAG_FIXEDWIDTH
-		? MAX (html_painter_get_pixel_size (painter) * table->specified_width, html_object_calc_min_width (o, painter))
+		? MAX (html_painter_get_pixel_size (painter) * table->specified_width, min_width)
 		: COLUMN_PREF (table, table->totalCols) + table->border * html_painter_get_pixel_size (painter);
 }
 
@@ -2160,7 +2166,7 @@ save_plain (HTMLObject *self,
 }
 
 static gboolean
-check_row_split (HTMLTable *table, gint r, gint *min_y)
+check_row_split (HTMLTable *table, HTMLPainter *painter, gint r, gint *min_y)
 {
 	HTMLTableCell *cell;
 	gboolean changed = FALSE;
@@ -2177,8 +2183,8 @@ check_row_split (HTMLTable *table, gint r, gint *min_y)
 		y2 = HTML_OBJECT (cell)->y + HTML_OBJECT (cell)->descent;
 
 		if (y1 <= *min_y && *min_y < y2) {
-			cs = html_object_check_page_split (HTML_OBJECT (cell), *min_y - y1) + y1;
-			/* printf ("min_y: %d y1: %d y2: %d --> cs=%d\n", *min_y, y1, y2, cs); */
+			cs = html_object_check_page_split (HTML_OBJECT (cell), painter, *min_y - y1) + y1;
+			/* printf ("r %d min_y: %d y1: %d y2: %d --> cs=%d\n", *min_y, y1, y2, cs); */
 
 			if (cs < *min_y) {
 				*min_y = cs;
@@ -2191,10 +2197,11 @@ check_row_split (HTMLTable *table, gint r, gint *min_y)
 }
 
 static gint
-check_page_split (HTMLObject *self, gint y)
+check_page_split (HTMLObject *self, HTMLPainter *painter, gint y)
 {
 	HTMLTable     *table;
-	gint r, min_y;
+	int pixel_size = html_painter_get_pixel_size (painter);
+	int r, min_y;
 
 	table = HTML_TABLE (self);
 	r     = to_index (bin_search_index (table->rowHeights, 0, table->totalRows, y), 0, table->totalRows - 1);
@@ -2207,7 +2214,7 @@ check_page_split (HTMLObject *self, gint y)
 	   return MIN (y, ROW_HEIGHT (table, table->totalRows)); */
 
 	min_y = MIN (y, ROW_HEIGHT (table, r+1));
-	while (check_row_split (table, r, &min_y));
+	while (check_row_split (table, painter, r, &min_y));
 
 	/* printf ("min_y=%d\n", min_y); */
 
